import { ElementRef, Renderer2 } from '@angular/core';
import { AbstractControl, AsyncValidator, AsyncValidatorFn, ControlValueAccessor, ValidationErrors } from '@angular/forms';
import { Observable } from 'rxjs';
export declare class FileInputAccessor implements ControlValueAccessor, AsyncValidator {
    private _renderer;
    private _elementRef;
    allowedExt: RegExp | string | string[];
    allowedTypes: RegExp | string | string[];
    size: number;
    withMeta: boolean;
    maxHeight: number;
    maxWidth: number;
    onChange: (_: any) => void;
    onTouched: () => void;
    validator: AsyncValidatorFn;
    constructor(_renderer: Renderer2, _elementRef: ElementRef);
    writeValue(value: any): void;
    registerOnChange(fn: (_: any) => {}): void;
    registerOnTouched(fn: () => {}): void;
    setDisabledState(isDisabled: boolean): void;
    validate(c: AbstractControl): Observable<ValidationErrors | null> | Promise<ValidationErrors | null>;
    /**
     * Generator method that I used to move the code for the AsyncValidator down here so it didn't
     * get in my way, way up there ^.
     * @returns {AsyncValidatorFn}
     */
    private generateAsyncValidator();
    /**
     * Generator method that returns an onChange handler
     * @returns {(_: ICustomFile[]) => void}
     * @param fn
     */
    private onChangeGenerator(fn);
    private generateRegExp(pattern);
    /**
     * The ICustomFile has a ReplaySubject property for text / image files that will emit
     * once the file has been loaded. Might get removed later since I haven't found a use for it yet.
     * @param {ICustomFile} f
     * @param {FileReader} fr
     */
    private generateFileMeta(f, fr);
    private setImage(f, fr);
    private setText(f, fr);
}
