import { Directive, ElementRef, forwardRef, HostListener, Input, Renderer2 } from '@angular/core';
import { NG_ASYNC_VALIDATORS, NG_VALUE_ACCESSOR } from '@angular/forms';
import { forkJoin, fromEventPattern, of, ReplaySubject } from 'rxjs';
import { first, map, shareReplay, take } from 'rxjs/operators';
var FileInputAccessor = /** @class */ (function () {
    function FileInputAccessor(_renderer, _elementRef) {
        this._renderer = _renderer;
        this._elementRef = _elementRef;
        this.onChange = function (_) { };
        this.onTouched = function () { };
        this.validator = this.generateAsyncValidator();
    }
    FileInputAccessor.prototype.writeValue = function (value) {
        this._renderer.setProperty(this._elementRef.nativeElement, 'value', null);
    };
    FileInputAccessor.prototype.registerOnChange = function (fn) {
        this.onChange = this.onChangeGenerator(fn);
    };
    FileInputAccessor.prototype.registerOnTouched = function (fn) { };
    FileInputAccessor.prototype.setDisabledState = function (isDisabled) {
        this._renderer.setProperty(this._elementRef.nativeElement, 'disabled', isDisabled);
    };
    FileInputAccessor.prototype.validate = function (c) {
        return this.validator(c);
    };
    /**
     * Generator method that I used to move the code for the AsyncValidator down here so it didn't
     * get in my way, way up there ^.
     * @returns {AsyncValidatorFn}
     */
    /**
         * Generator method that I used to move the code for the AsyncValidator down here so it didn't
         * get in my way, way up there ^.
         * @returns {AsyncValidatorFn}
         */
    FileInputAccessor.prototype.generateAsyncValidator = /**
         * Generator method that I used to move the code for the AsyncValidator down here so it didn't
         * get in my way, way up there ^.
         * @returns {AsyncValidatorFn}
         */
    function () {
        var _this = this;
        return function (c) {
            if (!c.value || !c.value.length || c.disabled)
                return of({});
            var errors = {};
            var loaders = [];
            var _loop_1 = function (f) {
                if (_this.size && _this.size < f.size) {
                    f.errors['fileSize'] = true;
                    errors['fileSize'] = true;
                }
                if (f.isImg && (_this.maxWidth || _this.maxHeight)) {
                    loaders.push(f.imgLoadReplay
                        .pipe(take(1), map(function (e) {
                        if (_this.maxWidth && f.imgWidth > _this.maxWidth) {
                            f.errors['imageWidth'] = true;
                            errors['imageWidth'] = true;
                        }
                        if (_this.maxHeight && f.imgHeight > _this.maxHeight) {
                            f.errors['imageHeight'] = true;
                            errors['imageHeight'] = true;
                        }
                        return e;
                    })));
                }
                if (!_this.allowedExt && !_this.allowedTypes)
                    return "continue";
                var extP = _this.generateRegExp(_this.allowedExt);
                var typeP = _this.generateRegExp(_this.allowedTypes);
                if (extP && !extP.test(f.name)) {
                    f.errors['fileExt'] = true;
                    errors['fileExt'] = true;
                }
                if (typeP && !typeP.test(f.type)) {
                    f.errors['fileType'] = true;
                    errors['fileType'] = true;
                }
            };
            for (var _i = 0, _a = c.value; _i < _a.length; _i++) {
                var f = _a[_i];
                _loop_1(f);
            }
            if (loaders.length) {
                return forkJoin.apply(void 0, loaders).pipe(map(function () { return errors; }));
            }
            return of(errors);
        };
    };
    ;
    /**
     * Generator method that returns an onChange handler
     * @returns {(_: ICustomFile[]) => void}
     * @param fn
     */
    /**
         * Generator method that returns an onChange handler
         * @returns {(_: ICustomFile[]) => void}
         * @param fn
         */
    FileInputAccessor.prototype.onChangeGenerator = /**
         * Generator method that returns an onChange handler
         * @returns {(_: ICustomFile[]) => void}
         * @param fn
         */
    function (fn) {
        var _this = this;
        return function (files) {
            var fileArr = [];
            for (var _i = 0, files_1 = files; _i < files_1.length; _i++) {
                var f = files_1[_i];
                if (_this.withMeta && FileReader) {
                    var fr = new FileReader();
                    _this.generateFileMeta(f, fr);
                }
                f.errors = {};
                fileArr.push(f);
            }
            fn(fileArr);
        };
    };
    ;
    FileInputAccessor.prototype.generateRegExp = function (pattern) {
        if (!pattern)
            return null;
        if (pattern instanceof RegExp) {
            return new RegExp(pattern);
        }
        else if (typeof pattern === 'string') {
            return new RegExp(pattern, 'ig');
        }
        else if (pattern instanceof Array) {
            return new RegExp("(" + pattern.join('|') + ")", 'ig');
        }
        return null;
    };
    /**
     * The ICustomFile has a ReplaySubject property for text / image files that will emit
     * once the file has been loaded. Might get removed later since I haven't found a use for it yet.
     * @param {ICustomFile} f
     * @param {FileReader} fr
     */
    /**
         * The ICustomFile has a ReplaySubject property for text / image files that will emit
         * once the file has been loaded. Might get removed later since I haven't found a use for it yet.
         * @param {ICustomFile} f
         * @param {FileReader} fr
         */
    FileInputAccessor.prototype.generateFileMeta = /**
         * The ICustomFile has a ReplaySubject property for text / image files that will emit
         * once the file has been loaded. Might get removed later since I haven't found a use for it yet.
         * @param {ICustomFile} f
         * @param {FileReader} fr
         */
    function (f, fr) {
        if (f.type.match(/text.*/)) {
            f.textLoadReplay = this.setText(f, fr);
        }
        else if (f.type.match(/image.*/)) {
            f.imgLoadReplay = this.setImage(f, fr);
        }
    };
    FileInputAccessor.prototype.setImage = function (f, fr) {
        f.isImg = true;
        var img = new Image();
        var imgLoadObs = fromEventPattern((function (handler) { return img.addEventListener('load', handler); }), (function (handler) { return img.removeEventListener('load', handler); })).pipe(take(1), shareReplay());
        var frLoadObs = fromEventPattern((function (handler) { return fr.addEventListener('load', handler); }), (function (handler) { return fr.removeEventListener('load', handler); })).pipe(take(1), shareReplay());
        var onloadReplay = new ReplaySubject(1);
        forkJoin(imgLoadObs, frLoadObs).pipe(first()).subscribe(onloadReplay);
        imgLoadObs.pipe(first()).subscribe(function () {
            f.imgHeight = img.height;
            f.imgWidth = img.width;
        });
        frLoadObs.pipe(first()).subscribe(function () {
            f.imgSrc = fr.result;
            img.src = fr.result;
        });
        fr.readAsDataURL(f);
        return onloadReplay;
    };
    FileInputAccessor.prototype.setText = function (f, fr) {
        var frLoadObs = fromEventPattern((function (handler) { return fr.addEventListener('load', handler); }), (function (handler) { return fr.removeEventListener('load', handler); })).pipe(take(1), shareReplay());
        var onloadReplay = new ReplaySubject(1);
        frLoadObs.subscribe(onloadReplay);
        frLoadObs.pipe(first()).subscribe(function () {
            f.textContent = fr.result;
        });
        fr.readAsText(f);
        return onloadReplay;
    };
    FileInputAccessor.decorators = [
        { type: Directive, args: [{
                    selector: 'input[type=file][formControl],input[type=file][formControlName],input[type=file][ngModel]',
                    providers: [
                        {
                            provide: NG_VALUE_ACCESSOR,
                            useExisting: forwardRef(function () { return FileInputAccessor; }),
                            multi: true
                        },
                        {
                            provide: NG_ASYNC_VALIDATORS,
                            useExisting: forwardRef(function () { return FileInputAccessor; }),
                            multi: true
                        }
                    ]
                },] },
    ];
    /** @nocollapse */
    FileInputAccessor.ctorParameters = function () { return [
        { type: Renderer2, },
        { type: ElementRef, },
    ]; };
    FileInputAccessor.propDecorators = {
        "allowedExt": [{ type: Input },],
        "allowedTypes": [{ type: Input },],
        "size": [{ type: Input },],
        "withMeta": [{ type: Input },],
        "maxHeight": [{ type: Input },],
        "maxWidth": [{ type: Input },],
        "onChange": [{ type: HostListener, args: ['change', ['$event.target.files'],] },],
        "onTouched": [{ type: HostListener, args: ['blur',] },],
    };
    return FileInputAccessor;
}());
export { FileInputAccessor };
//# sourceMappingURL=file-input-accessor.js.map